//#######################################################################################################################
//#
//# БЕГУЩАЯ СТРОКА 48x8 пикселей, с функцией отображения времени и температуры.
//# Контроллер ATMEGA8
//#
//#######################################################################################################################
#include "LedBoard_8x48_define.h"				//включаемые файлы
#include "LedBoard_8x48_subroutines.h"
#include "LedBoard_8x48_interrupt.h"


//-----------------------------------------------------------------------------------------------------------------------
//массив сообщений для вывода на дисплей
char TxtTitle	[] PROGMEM = " Atmega8 8x48 Ledmбtrix уra йs kьlsх belsх hхmйrх! Sohase sajnбld az idхt, amit arra fordнtasz, hogy jуl tedd amit teszel!!!   Most nйzzьk meg a benti hхmйrsйkletet! >> ";
char TxtTitle1	[] PROGMEM = "Kint vajon mit mutat a hхmйrх ? >> ";
char TxtTitle2	[] PROGMEM = "A pohбr mindig fйlig van tцltve, s nem azon kell keseregni, hogy miйrt nincs tele. Annak kell цrьlni, hogy nem ьres.( Diana Dorth ) Szйp napot kнvбnok mindenkinek!!   * * Varga Lajos * * ";

//**************************************************************************
//**************************************************************************
//** доступные функции:
//**
//** void LedBoardIntPrint 			(integer, IntPos, IntLength, DotPosition);	//Вывод 16 битного числа, со знаком и десятичной точкой
//** void LedBoardMessageShow 		(*pAdrTxt);									//Вывод в строку надписи 
//** void LedBoardMessageScrollLeft	(*pAdrTxt);									//Вывод в строку надписи со сдвигом влево
//** void LedBoardTermoShow 		(Position, NumDev);							//Вывод в строку данных температуры
//** void LedBoardTermoScrollLeft 	(NumDev);									//Вывод в строку данных температуры со сдвигом влево
//** void LedBoardClockShow 		(Position);									//Вывод в строку времени часов
//** void LedBoardClockScrollLeft 	();											//Вывод в строку времени часов со сдвигом влево
//** void LedBoardClearScrollDown 	();											//Очистка строки сдвигом вниз
//** void LedBoardClearScrollLeft 	();											//Очистка строки сдвигом влево
//**************************************************************************
//**************************************************************************


//ГЛАВНАЯ ФУНКЦИЯ
int main (void)
{
	MainInit ();	
	LedBoardHorScrollSpeed = 30;

	LedBoardClockScrollLeft ();
	Delay (1);
	LedBoardClearScrollLeft ();

	LedBoardMessageScrollLeft (TxtTitle);
	LedBoardTermoScrollLeft (0);
	Delay (1);
	LedBoardClearScrollDown ();
	
	LedBoardMessageScrollLeft (TxtTitle1);
	LedBoardTermoScrollLeft (1);
	Delay (1);
	LedBoardClearScrollDown ();
	LedBoardClockScrollLeft ();

	while (1)
{	
menu();
}


/*	while (1){
		char BtnMask = BtnFlags;
		BtnFlags =0;		

		if (BtnMask == ShortUp)
			LedBoardMessageShow (TxtUp);

		if (BtnMask == ShortDown)
			LedBoardMessageShow (TxtDown);

		if (BtnMask == ShortLeft)
			LedBoardMessageShow (TxtLeft);

		if (BtnMask == ShortRight)
			LedBoardMessageShow (TxtRight);
	}*/

}


//#######################################################################################################################
//#
//#	THE END
//#
//#######################################################################################################################



